#*********************************************************************************************
#*********************************************************************************************
#**
#**  File     :  /maps/Solo ACU/Solo ACU_script.lua
#**
#**  Author(s):  Manimal (scripters) + Saya (help) + ertwyu, BennyMoore, KrimsonStorm (game concept)
#**
#**  Version  :  2
#**
#**  Date     :  24/08/2007
#**
#**  Summary  :  Solo ACU Map Script
#**
#**********************************************************************************************
#**********************************************************************************************


--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   Local Definitions                                                         ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--


local ScenarioUtils = import('/lua/sim/ScenarioUtilities.lua')
local ScenarioFramework = import( '/lua/ScenarioFramework.lua' )
local Utilities = import('/lua/Utilities.lua')



--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   Misc Global Definitions                                                   ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--


attackUBC = false
defensUBC = false

ScenarioInfo.Attacker = 1
ScenarioInfo.Defender = 2

A_Air_Transport_1 = false
A_Air_Spies_1     = { 
                      Air_Spy_01 = false,
                      Air_Spy_02 = false,
                      Air_Spy_03 = false,
                      Air_Spy_04 = false,
                      Air_Spy_05 = false,
                      Air_Spy_06 = false,
                      Air_Spy_07 = false,
                      Air_Spy_08 = false,
                      Air_Spy_09 = false,
                      Air_Spy_10 = false
                    }





--[[   ubcsTable                                                                 ]]--
--[[   { name, enemi, unitID,                                                    ]]--
--[[          EnhancementsList, EnhRestrictionsList, BuildRestrictionsList }     ]]--
--[[                                                                             ]]--
--[[   Do NOT erase any line in this ubcsTable !                                 ]]--
--[[   Add a comment sign instead for an evenutal future use                     ]]--

ubcsTable = {

    { name = 'ATTACKER',
      enemi = 'DEFENDER',
      unitId = 'url0001', 
      EnhancementsList      = { 'AdvancedEngineering',
                                'CoolingUpgrade',
                                'MicrowaveLaserGenerator',
                                'ResourceAllocation',
                                'T3Engineering'
                              },
      EnhRestrictionsList   = { 'NaniteTorpedoTube',
                                'StealthGenerator'
                              },
      BuildRestrictionsList = { categories.AEON,
                                categories.UEF,
                                categories.NAVAL,
                                categories.EXPERIMENTAL,
                                categories.ura0102,  # Interceptor
                                categories.ura0103,  # Attack Bomber
                                categories.ura0203,  # Gunship
                                categories.ura0204,  # Torpedo Bomber
                                categories.dra0202,  # Bomber Corsair
                                categories.ura0303,  # Air Superiority Fighter
                                categories.ura0304,  # Strategic Bomber
                                categories.urb0304,  # Quantum Gateway
                                categories.urb1102,  # Hydrocarbon Power Plant
                                categories.urb2108,  # Tactical Missile Launcher
                                categories.urb2109,  # Tech 1 Torpedo Launcher
                                categories.urb2205,  # Tech 2 Torpedo Launcher
                                categories.urb2301,  # Tech 2 Point Defense (Heavy Gun Tower)
                                categories.urb2302,  # Long Range Artillery
                                categories.urb2303,  # Light Artillery
                                categories.urb3102,  # Tech 1 Sonar System
                                categories.urb3104,  # Omni Sensor Array
                                categories.urb3202,  # Tech 2 Sonar System
                                categories.urb4203,  # Stealth Field Generator
                                categories.urb5101,  # Wall Section
                                categories.urb5103,  # Quantum Gate Beacon
                                categories.url0103,  # Mobile Light Artillery
                                categories.url0106,  # Light Assault Bot
                                categories.url0107,  # Heavy Assault Bot
                                categories.url0111,  # Mobile Missile Launcher
                                categories.url0202,  # Heavy Tank
                                categories.url0203,  # Amphibious Tank
                                categories.url0301,  # Support Command Unit
                                categories.url0303,  # Siege Assault Bot
                                categories.url0304,  # Mobile Heavy Artillery
                                categories.url0306,  # Mobile Stealth Field System
                                categories.url5103,  # Quantum Gate Beacon
                                categories.urs0305   # Sonar Platform
                              }
    },

    { name = 'DEFENDER', 
      enemi = 'ATTACKER',
      unitId = 'uel0001', 
      EnhancementsList      = { 'AdvancedEngineering'},
      EnhRestrictionsList   = { 'DamageStablization',
                                'HeavyAntiMatterCannon',
                                'T3Engineering',
                                'TacticalNukeMissile'
                              },
      BuildRestrictionsList = { categories.AEON,
                                categories.CYBRAN,
                                categories.NAVAL,
                                categories.TECH3,
                                categories.EXPERIMENTAL,
                                categories.ueb1102,  # Hydrocarbon Power Plant
                                categories.ueb1201,  # Tech 2 Power Plant
                                categories.ueb2109,  # Tech 1 Torpedo Launcher
                                categories.ueb2205,  # Tech 2 Torpedo Launcher
                                categories.ueb2301,  # Tech 2 Point Defense (Heavy Gun Tower)
                                categories.ueb2302,  # Long Range Artillery
                                categories.ueb2303,  # Light Artillery
                                categories.ueb3102,  # Tech 1 Sonar System
                                categories.ueb3202,  # Tech 2 Sonar System
                                categories.ueb5101,  # Wall Section
                                categories.uel0307   # Mobile Shield Generator
                              }
    }

} 
# end of ubcsTable


unitsTable = {
              { name   = 'Air_Transport_1', 
                enemi  = 'DEFENDER',
                unitId = 'ura0104'
              },
              { name   = 'Air_Spies_1', 
                enemi  = 'DEFENDER',
                unitId = 'ura0302'
              }
             }





--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   Local Tuning Variables                                                    ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--


# The players's unit cap
local AttackerUnitCap = 500
local DefenderUnitCap = 500





--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   Misc Local Definitions                                                    ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--


local theAttacker = ScenarioInfo.Attacker
local theDefender = ScenarioInfo.Defender






--[[ *************************************************************************** ]]--
--[[ *************************************************************************** ]]--
--[[                                                                             ]]--
--[[   STARTER FUNCTIONS                                                         ]]--
--[[                                                                             ]]--
--[[ *************************************************************************** ]]--
--[[ *************************************************************************** ]]--





--[[                                                                             ]]--
--[[   OnPopulate()                                                              ]]--
--[[                                                                             ]]--
--[[   alliances and unit spawning go in there so you dont get conflicts and     ]]--
--[[   weird behaviours with initializearmies                                    ]]--
--[[                                                                             ]]--

function OnPopulate()

    # IMPORTANT NOTE :
    # ~~~~~~~~~~~~~~~~
    # The Controls for Players MUST BE LOCKED until the ACU is
    # transported and unloaded to his DropZone.

    # Lock the Controls until ALL Units are READY
    LockInput()

    # Custom Initialization of Armies
    InitializeArmies()


    --[[  GENERAL SETUP                                                          ]]--
    --[[                                                                         ]]--
    --[[  Scores, UnitCap, Alliances,                                            ]]--


    --[[  Fix Scenario's Options  ]]--

    ScenarioInfo.Options.Victory          = 'assassination'
    ScenarioInfo.Options.TeamSpawn        = 'fixed'
    ScenarioInfo.Options.TeamLock         = 'locked'
    ScenarioInfo.Options.FogOfWar         = 'explored' 
    ScenarioInfo.Options.CivilianAlliance = 'neutral' 
    ScenarioInfo.Options.CheatsEnabled    = 'false' 
    ScenarioInfo.Options.NoRushOption     = 'Off' 
    ScenarioInfo.Options.PrebuiltUnits    = 'Off' 
    ScenarioInfo.Options.UnitCap          = '500' 

    --[[  Set the playable AREA  ]]--

    #ScenarioFramework.SetPlayableArea( Rect(0, 0, 512, 512) )

    --[[  Fix players's Unit Cap, NO EXTRA UNITS ALLOWED  ]]--

    SetIgnoreArmyUnitCap( theAttacker, false )
    SetIgnoreArmyUnitCap( theDefender, false )
    SetArmyUnitCap( theAttacker, AttackerUnitCap )
    SetArmyUnitCap( theDefender, DefenderUnitCap )

    --[[  Set ALL Players's ALLIANCES  ]]--

    SetAlliance( ubcsTable[theAttacker].name, ubcsTable[theDefender].name, 'Enemy' )


    --[[  Transport UBC jusqu'� la Drop Zone  ]]--

    if attackUBC != nil then
        TransportUBCversDropZone( attackUBC )
    else
        LOG("MANIMAL:  TransportUBCversDropZone of attackUBC is not possible for player ".. iArmy .. " " .. strArmy .. " .")
        UnlockInput()
    end


    --[[  Envoyer Patrouille AirSpies autour de la Drop Zone  ]]--

    LOG("MANIMAL:  DEBUG Avant Creation de A_Air_Spies_1 .")
    AirSpiesPatrouilleDropZone()


end






--[[                                                                             ]]--
--[[   OnStart(self)                                                             ]]--
--[[                                                                             ]]--
--[[   Game mechanisms are there                                                 ]]--
--[[                                                                             ]]--

function OnStart(self)

    --[[  LET's ROCK !  ]]--

    #ForkThread( StartSoloACU() )

end






--[[ *************************************************************************** ]]--
--[[ *************************************************************************** ]]--
--[[                                                                             ]]--
--[[  OTHER FUNCTIONS                                                            ]]--
--[[                                                                             ]]--
--[[ *************************************************************************** ]]--
--[[ *************************************************************************** ]]--




--[[                                                                             ]]--
--[[   InitializeArmies()                                                        ]]--
--[[                                                                             ]]--
--[[   Spawn CUSTOM COMMANDERS and their ARMIES                                  ]]--
--[[                                                                             ]]--

function InitializeArmies()

    local tblGroups = {}
    local tblArmy = ListArmies()

    for iArmy, strArmy in pairs(tblArmy) do

        local tblData = Scenario.Armies[strArmy]

        if tblData then
		
            SetArmyEconomy( strArmy, tblData.Economy.mass, tblData.Economy.energy)

            --[[  Spawn Upgraded ACUs  ]]--

            local cdrUnit = false
			
            cdrUnit = SpawnUpgradedUBC ( strArmy, iArmy ) 

            if cdrUnit != nil then
                LOG("MANIMAL:  Successful SPAWN of attackUBC for player ".. iArmy .. " " .. strArmy .. " .")
                if strArmy == ubcsTable[theAttacker].name then
                    attackUBC = cdrUnit
                elseif strArmy == ubcsTable[theDefender].name then
                    defensUBC = cdrUnit
                end
            end

            --[[  PLAYERS'S RESTRICTIONS                                      ]]--
            --[[  Take away units that the players shouldn't have access to   ]]--

            if ubcsTable[iArmy].BuildRestrictionsList then 
                for i,catRestrict in ubcsTable[iArmy].BuildRestrictionsList do
                    LOG("MANIMAL:  AddRestriction at index=" .. i .. " for player ".. iArmy .. " " .. strArmy .. " .")
                    ScenarioFramework.AddRestriction( strArmy, catRestrict )
                end
            end

            --[[  Pre-Built UNITS/BUILDINGS  ]]--

            local preBuilt = false

            preBuilt =  ScenarioUtils.CreateArmySubGroup(strArmy, 'INITIAL')


            --[[  Spawn Wreckage etc...  ]]--
            --[[
            local wreckageGroup = FindUnitGroup('WRECKAGE', Scenario.Armies[strArmy].Units)

            if wreckageGroup then

                local platoonList, tblResult, treeResult = CreatePlatoons(strArmy, wreckageGroup )

                for num,unit in tblResult do
                    unit:CreateWreckageProp(0)
                    unit:Destroy()
                end

            end   #end IF wreckageGroup
            --]]


        end   #end IF tblData

    end   #end FOR 


    return tblGroups

end




--[[                                                                             ]]--
--[[   TransportUBCversDropZone( cdrUnit )                                       ]]--
--[[                                                                             ]]--
--[[   Load and Transport the Attacker's ACU to the Drop Zone                    ]]--
--[[                                                                             ]]--

function TransportUBCversDropZone(cdrUnit)

    --[[  Spawn Air Transport for Attacker ACU  ]]--

    local spawnPosLoc    = mPos( 'DZ_P0' )
    local origMarkerPos  = ScenarioUtils.MarkerToPosition( 'DZ_P0' )
    local destMarkerPos  = ScenarioUtils.MarkerToPosition( 'DZ_P1' )
			
    A_Air_Transport_1 = CreateUnitHPR( unitsTable[1].unitId, ubcsTable[theAttacker].name, spawnPosLoc[1], spawnPosLoc[2], spawnPosLoc[3], 0, 0, 0 )

    # successful spawn so the transport can be loaded up !
    if A_Air_Transport_1 != nil then

        LOG("MANIMAL:  Successful SPAWN of A_Air_Transport_1 .")

        --[[  LOADING ATTACKER UP IN AIR TRANSPORT  ]]--

        if not cdrUnit:IsDead() then
            # Automatically attaches <units> to attach points on <transports>
            # function AttachUnitsToTransports(units, transports)
            # units is a set of units, transports is a set of transports
            ScenarioFramework.AttachUnitsToTransports( { cdrUnit }, { A_Air_Transport_1 } )
            LOG("MANIMAL:  ACU is ATTACHED to A_Air_Transport_1 for player .")
        end


        --[[  UNLOADING ATTACKER FROM AIR TRANSPORT  ]]--

        if not cdrUnit:IsDead() then
            # IssueTransportUnload( transports, landingLocation )
            # transports is a set of transports, landingLocation = is a marker's position
            IssueTransportUnload( { A_Air_Transport_1 }, destMarkerPos )
            LOG("MANIMAL:  ACU is UNLOADED from A_Air_Transport_1 .")

            # IMPORTANT NOTE :
            # ~~~~~~~~~~~~~~~~
            # The Controls for Players MUST BE LOCKED until the ACU is unloaded

            # When ACU is unloaded from transport, Unlock the Controls for Players
            if not A_Air_Transport_1:IsDead() then

                # SI CE TEST NE FONCTIONNE PAS, ALORS TESTER PRESENCE ACU DANS DROPZONE1
                # Wait for ACU to be unloaded from transport (estimated time)
                if not cdrUnit:IsUnitState('Attached') then
                    UnlockInput()
                end

            end

        end

    end

end





--[[                                                                             ]]--
--[[  AirSpiesPatrouilleDropZone()                                               ]]--
--[[                                                                             ]]--
--[[  Patrouille AirSpies autour de la Drop Zone                                 ]]--
--[[                                                                             ]]--

function AirSpiesPatrouilleDropZone()

    --[[  Spawn Air Spies for Attacker ACU  ]]--

    local spawnPosLoc   = mPos( 'DZ_P0_1' )
    local origMarkerPos = ScenarioUtils.MarkerToPosition( 'DZ_P0_1' )
    local destMarkerPos = ScenarioUtils.MarkerToPosition( 'DZ_P1' )
    local attackChain   = 'Drop_Zone_1_Patrol'
			
    # Creer 10 SpyPlanes Cybran Tech 2
    local unitcount = 1
    local Air_Spy = false
    while  unitcount != 11  do
        Air_Spy = A_Air_Spies_1[unitcount]
        if not Air_Spy then
            Air_Spy = CreateUnitHPR( unitsTable[2].unitId, ubcsTable[theAttacker].name, spawnPosLoc[1], spawnPosLoc[2], spawnPosLoc[3], 0, 0, 0 )
            if Air_Spy != nil then
                A_Air_Spies_1[unitcount] = Air_Spy
                unitcount = unitcount + 1
            end
        end
    end

    # successful spawn so the Spies can go to Patrol !
    if A_Air_Spies_1 != nil then

        LOG("MANIMAL:  Successful SPAWN of A_Air_Spies_1 .")

        --[[  Patrouille AirSpies autour de la Drop Zone  ]]--

        local attackLocs = ScenarioUtils.ChainToPositions(attackChain)

        for k,v in attackLocs do
            LOG("MANIMAL:  A_Air_Spies_1 en Patrouille sur Zone=Drop_Zone_1_Patrol .")
            IssuePatrol( A_Air_Spies_1, v )
        end

    end

end





--[[                                                                             ]]--
--[[  StartSoloACU()                                                             ]]--
--[[                                                                             ]]--
--[[  Start the Custom Scenario                                                  ]]--
--[[                                                                             ]]--

function StartSoloACU()

end




--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   UTILITY FUNCTIONS                                                         ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--




--[[                                                                             ]]--
--[[   GetMarker(name)                                                           ]]--
--[[                                                                             ]]--
--[[   get marker given a name from map_save done in map editor                  ]]--
--[[   you can the use marker.position to get a location in the {x,y,z} format   ]]--
--[[                                                                             ]]--

function GetMarker(name)
    return Scenario.MasterChain._MASTERCHAIN_.Markers[name]
end



--[[                                                                             ]]--
--[[   mPos(name)                                                                ]]--
--[[                                                                             ]]--
--[[   Get Marker position                                                       ]]--
--[[                                                                             ]]--

function mPos(name)
    return GetMarker(name).position
end



--[[                                                                             ]]--
--[[   strEnemi(iArmy)                                                           ]]--
--[[                                                                             ]]--
--[[   Returns the enemi of the current Army                                     ]]--
--[[                                                                             ]]--

function strEnemi(iArmy)
    return ubcsTable[iArmy].enemi
end




--[[                                                                             ]]--
--[[   SpawnUpgradedUBC ( strArmy, iArmy )                                       ]]--
--[[                                                                             ]]--
--[[   Spawn an Upgraded ACU                                                     ]]--
--[[                                                                             ]]--

function SpawnUpgradedUBC ( strArmy, iArmy )

    local ubcId = ''       # ACU Identifier
    local cdrUnit = false

    ubcId = ubcsTable[iArmy].unitId

    if ubcId != '' then  # ACU Identifier is valid

        # spawn uef commander
        cdrUnit = CreateUnitHPR( ubcId, strArmy, mPos(strArmy)[1], mPos(strArmy)[2],mPos(strArmy)[3], 0,0,0)

        # successful spawn
        if cdrUnit != nil then

            --[[  DO ENHANCEMENTS  ]]--

            if ubcsTable[iArmy].EnhancementsList then 
                for i,enhancement in ubcsTable[iArmy].EnhancementsList do
                    LOG("MANIMAL:  ACU Enhancement at index=" .. i .. ", enhancement=" .. enhancement .. " for player ".. iArmy .. " " .. strArmy .. " .")
                    cdrUnit:CreateEnhancement(enhancement)
                end
            end

            # customize name as explicitly as possible
            cdrUnit:SetCustomName( ArmyBrains[iArmy].Nickname .. " - " .. strArmy )
				
            # annihilation type victory condition
            cdrUnit.OldOnKilled = cdrUnit.OnKilled

            # action when killed
            cdrUnit.OnKilled = function(self, instigator, type, overkillRatio)
                GetArmyBrain(strArmy):OnDefeat()
                GetArmyBrain(strEnemi(iArmy)):OnVictory()
                cdrUnit.OldOnKilled(self, instigator, type, overkillRatio)	
            end
				
            --[[  DO RESTRICTIONS  ]]--

            if ubcsTable[iArmy].EnhRestrictionsList then 
                for i,restriction in ubcsTable[iArmy].EnhRestrictionsList do
                    LOG("MANIMAL:  ACU RestrictEnhancement at index=" .. i .. ", restriction=" .. restriction .. " for player ".. iArmy .. " " .. strArmy .. " .")
                    ScenarioFramework.RestrictEnhancements(restriction)
                end
            end

        else
            LOG("FAILED: Unit Creation (ACU) for player ".. iArmy .. " " .. strArmy .. " .")
        end   #end IF cdrUnit

        # play warp in effect
        #cdrUnit:PlayCommanderWarpInEffect()

   end   #end IF ubcId

   return cdrUnit

end





 